
!function ($) {
    "use strict";

    function getPubsPack(id) {
        var res = [];
        Object.keys(nbPubs).forEach(function(key) {
            var idPack = key.split("#")[1];
            if(idPack == id) {
                res.push({
                    date: key.split("#")[0],
                    nb: nbPubs[key]
                });
            }
        }, this);

        return res;
    }

    function formatDate(date) {
        var dateStr = "";
        var month = "";
        if (date.getDate() < 10)
            dateStr = "0" + date.getDate();
        else
            dateStr = date.getDate();

        if (date.getMonth() < 9)
            month = "0" + (date.getMonth() + 1);
        else
            month = date.getMonth();

        return date.getFullYear() + "-" + month + "-" + dateStr;
    }

    var pubManager = {
        pubs: {},
        pagePub: $("#pagePub"),
        pageEmplacement: $("#page-emplacement"),
        pagePaiement: $("#pagePaiement"),
        pageCheckout: $("#pageCheckout"),
        btnValidate: $("#validate"),
        btnEmplacement: $(".btn-emplacement"),
        btnPub: $(".btn-pub"),
        commandeContainer: $("#commande-container"),
        priceContainer: $("#price-container"),
        getPubs: getPubs,
        addPub: addPub,
        remove: remove,
        update: update,
        getPrice: getPrice,
        init: init,
        initPub: initPub,
        initPaiement: initPaiement,
        initCheckout: initCheckout,
        save: save,
        getPubsPack : getPubsPack
    }

    function save() {
        var self = this;
        if (admin)
            var route = "admin_add_pub";
        else
            var route = "add_pub";

        addCache();
        $.ajax({
            type: "POST",
            url: Routing.generate(route, { type: type, _locale: locale }),
            data: { pubs: self.getPubs() },
            dataType: "JSON",
            success: function (response) {
                removeCache();
                if (admin)
                    window.location.href = Routing.generate("admin_pubs", {'_locale' : locale});
                else
                    window.location.href = Routing.generate("myadmin_pubs", {'_locale' : locale});
            }
        });
    }

    pubManager.init();

    function init() {
        var self = this;
        this.pageEmplacement.fadeIn();
        this.pagePub.fadeOut();
        this.pagePaiement.hide();
        this.pageCheckout.hide();
        this.btnPub.off("click").click(function () {
            if (!$(".page.selected").length) {
                alert("Veuillez selectionner une emplacement");
                return false;
            }

            self.update();

            self.initPub();
        });
    }

    function initPub() {
        var self = this;
        $("#emplacementTxt").html($(".page.selected").html());
        this.pageEmplacement.hide();
        this.pagePub.fadeIn();
        this.pagePaiement.hide();
        this.pageCheckout.hide();
        this.btnEmplacement.off("click").click(function () {
            self.init();
        });
        this.btnValidate.off("click").click(function () {
            self.initPaiement();
        });
    }

    function initPaiement() {
        var self = this;
        this.pagePaiement.fadeIn();
        this.pagePub.fadeOut();
        this.pageCheckout.fadeOut();

        $(".btn-back").off("click").click(function (e) {
            self.init();
        });

        $(".btn-validate").off("click").click(function (e) {
            if (admin || pubManager.getPrice() == 0) {
                self.save();
                return false;
            }
            self.initCheckout();
        });

        var rows = "";
        var pubs = this.getPubs();
        $.each(pubs, function (index, pub) {
            rows += `
                <tr>
                    <td>`+ pub.date + `</td>
                    <td>`+ pub.name + `</td>
                    <td>`+ pub.type + `</td>
                    <td>`+ pub.place + `</td>
                    <td>`+ ((pub.price == 0 ) ? "Credit pub" : pub.price +" €") + `</td>
                    <td><button class="btn btn-sm btn-danger btn-removep-pub" data-id="`+ pub.id + `"><i class="fa fa-trash"></i></button></td>
                </tr>
             `;
        });

        $("#totalPrice").html(this.getPrice() + " €");
        $("#paiement-table tbody").html(rows);

        $("#paiement-table .btn-removep-pub").click(function (e) {
            var id = $(this).data("id");
            self.remove(id);
            $(this).parent().parent().remove();
            $("#totalPrice").html(self.getPrice() + " €");
        });
    }

    function initCheckout() {
        var self = this;
        this.pageCheckout.fadeIn();
        this.pagePub.fadeOut();
        this.pagePaiement.fadeOut();

        $(".btn-save").off("click").click(function (e) {
            self.save();
        });

        $("#btn-checkout").click(function (e) {
            $("#modalPaiement").modal("show");
        });

        var rows = "";
        var pubs = this.getPubs();
        $.each(pubs, function (index, pub) {
            rows += `
                <tr>
                    <td>`+ pub.date + `</td>
                    <td>`+ pub.name + `</td>
                    <td>`+ pub.type + `</td>
                    <td>Page `+ pub.place + `</td>
                    <td>`+ pub.price + ` €</td>
                    <td></td>
                </tr>
             `;
        });

        $("#checkout-table .totalPrice").html(this.getPrice() + " €");
        $("#checkout-table tbody").html(rows);

        $("#acceptCondition").change(function () {
            $("#payment-not-allowed").attr("hidden", $(this).is(":checked"));
        });
    }

    if (!admin) {
        if (typeof paypal != "undefined") {
            paypal.Button.render({
                env: env, // Optional: specify 'sandbox' ('production') environment
                locale: 'fr_FR',
                payment: function () {
                    var env = this.props.env;
                    var client = this.props.client;
                    return paypal.rest.payment.create(env, client, {
                        transactions: [
                            {
                                amount: { total: parseFloat(pubManager.getPrice().toFixed(2)), currency: 'EUR' }
                            }
                        ]
                    });
                },
                commit: true,
                client: {
                    sandbox: paypal_client_id_sandbox,
                    production: paypal_client_id
                },
                onAuthorize: function (data, actions) {
                    return actions.payment.execute().then(function () {
                        pubManager.save();
                        $("#modalPaiement").modal("hide");
                    });
                }
            }, '#paypal-button');
        }
    }

    function update() {
        var self = this;
        var pubs = this.getPubs();
        var price = this.getPrice();

        var rows = "";
        $.map(pubs, function (pub, index) {
            rows += `
            <li style="margin-bottom: 15px;"> <b>`+ pub.date + `</b> <br />`+ pub.name + ` <span style="cursor: default;" class="btn btn-danger btn-xs btn-removeData pull-right" data-id="` + pub.id + `"><i class="fa fa-trash"></i></span>  </li>
            `;
        });

        var pubs = pubManager.getPubsPack($(".page.selected").data("id"));
        // $("td.fc-widget-content b").remove();
        $.map(pubs, function (pub, index) {
            var dateStr = formatDate(new Date(pub.date));
            var nb = pub.nb;
            $("td.fc-widget-content[data-date='" + dateStr + "']").css("position", "relative");
            var rest = 10 - nb;
            $("td.fc-widget-content[data-date='" + dateStr + "'] .nbPub").remove();
            var date = new Date(dateStr);
            var now = new Date();
            now.setDate(now.getDate() - 1);
            
            if(date >= now){
                if (rest == 0)
                    $("td.fc-widget-content[data-date='" + dateStr + "']").append("<b class='nbPub'>Complet</b>");
                else
                    $("td.fc-widget-content[data-date='" + dateStr + "']").append("<b class='nbPub'>" + rest +"</b>");
            }
        });


        this.commandeContainer.html(rows);
        this.priceContainer.html("Total : " + price + " €");
        this.btnValidate.attr("hidden", this.pubs.length == 0);

        $(".btn-removeData").off("click").click(function (e) {
            var id = $(this).data("id");
            self.remove(id);
        })
    }

    function getPrice() {
        var total = 0;
        $.each(this.getPubs(), function (index, pub) {

            total += pub.price;
        });

        return total;
    }

    function getPubs() {
        var self = this;
        var res = [];
        $.map(Object.keys(self.pubs), function (key, index) {
            res.push(self.pubs[key]);
        });

        return res;
    }

    function addPub(pub) {
        this.pubs[pub.id] = pub;
        var date = formatDate(new Date(pub.dateEn));
        if(nbPubs[date+"#"+$(".page.selected").data("id")]) {
            nbPubs[date+"#"+$(".page.selected").data("id")] = nbPubs[date+"#"+$(".page.selected").data("id")] + 1;
        } else {
            nbPubs[date+"#"+$(".page.selected").data("id")] = 1;
        }
        
        this.update();
    }

    function remove(id) {
        var date = formatDate(new Date(this.pubs[id].dateEn));
        var nb = nbPubs[date];
        nbPubs[date] = --nb;

        delete this.pubs[id];
        nbPubs[date+"#"+$(".page.selected").data("id")] = nbPubs[date+"#"+$(".page.selected").data("id")] - 1;
        this.update();
    }

    var CalendarApp = function () {
        this.$body = $("body")
        this.$calendar = $('#calendar'),
            this.$event = ('#calendar-events div.calendar-events'),
            this.$categoryForm = $('#add-new-event form'),
            this.$extEvents = $('#calendar-events'),
            this.$modal = $('#my-event'),
            this.$saveCategoryBtn = $('.save-category'),
            this.$calendarObj = null
    };


    /* on drop */
    CalendarApp.prototype.onDrop = function (eventObj, date) {
        return false;
        var $this = this;
        // retrieve the dropped element's stored Event Object
        var originalEventObject = eventObj.data('eventObject');
        var $categoryClass = eventObj.attr('data-class');
        // we need to copy it, so that multiple events don't have a reference to the same object
        var copiedEventObject = $.extend({}, originalEventObject);
        // assign it the date that was reported
        copiedEventObject.start = date;
        if ($categoryClass)
            copiedEventObject['className'] = [$categoryClass];
        // render the event on the calendar
        $this.$calendar.fullCalendar('renderEvent', copiedEventObject, true);
        // is the "remove after drop" checkbox checked?
        if ($('#drop-remove').is(':checked')) {
            // if so, remove the element from the "Draggable Events" list
            eventObj.remove();
        }
    },
        /* on click on event */
        CalendarApp.prototype.onEventClick = function (calEvent, jsEvent, view) {
            return false;
            var $this = this;
            var form = $("<form></form>");
            form.append("<p>Voulez vous vraiement supprimé ?</p>");
            $this.$modal.modal({
                backdrop: 'static'
            });

            $this.$modal.find('.delete-event').show().end().find('.save-event').hide().end().find('.modal-body').empty().prepend(form).end().find('.delete-event').unbind('click').click(function () {
                $this.$calendarObj.fullCalendar('removeEvents', function (ev) {
                    if (ev._id == calEvent._id)
                        pubManager.remove(ev._id);

                    return (ev._id == calEvent._id);
                });
                $this.$modal.modal('hide');
            });
            $this.$modal.find('form').on('submit', function () {
                // calEvent.title = form.find("input[type=text]").val();
                // $this.$calendarObj.fullCalendar('updateEvent', calEvent);
                // $this.$modal.modal('hide');
                return false;
            });
        },
        /* on select */
        CalendarApp.prototype.onSelect = function (start, end, allDay) {
            var check = new Date(start);
            var today = new Date();
            var dateStr = formatDate(check);
            var selected = $(".page.selected");
            var validDate = new Date();
            validDate.setDate(validDate.getDate() - 1);

            if (check <= validDate) {
                return false;
            }

            if (!selected.length) {
                alert("Veuillez selectionner une emplacement");
                return false;
            }

            if (nbPubs[dateStr+"#"+$(".page.selected").data("id")] == 10) {
                alert("La date est complet");
                return false;
            }
            var $this = this;

            $this.$modal.modal({
                backdrop: 'static'
            });
            var form = $("<form></form>");
            form.append("<div class='row'></div>");
            form.find(".row")
                .append("<div hidden class='col-md-6'><div class='form-group'><label class='control-label'>Event Name</label><input class='form-control' value='Publicité " + type + "' type='text' name='title'/></div></div>")
                .append("<div hidden class='col-md-6'><div class='form-group'><label class='control-label'>Category</label><select class='form-control' name='category'></select></div></div>")
                .find("select[name='category']")
                .append("<option value='bg-purple'>Purple</option>")
                .append("<option value='bg-primary'>Primary</option>")
                .append("<option value='bg-pink'>Pink</option>")
                .append("<option value='bg-info'>Info</option>")
                .append("<option value='bg-warning'>Warning</option></div></div>");

            switch (type) {
                case "music":
                    var musicsTable = `
                        <div class="col-lg-12" style="margin-top: 25px;">
                        <table style="background: transparent; color:white;" id="tableMusic" class="table table-bordered">
                            <thead>
                                <tr>
                                    <th></th>
                                    <th></th>
                                    <th>Nom</th>
                                    <th>Durée</th>
                                </tr>
                            </thead>
                            <tbody>
                    `;

                    $.each(musics, function (index, music) {
                        musicsTable += `
                        <tr>
                            <td><input type="checkbox" class="check check-music" data-name="`+ music.name + `" data-id="` + music.id + `"</td>
                            <td><img src="`+ music.cover + `" width="48px"/></td>
                            <td>`+ music.name + `</td>
                            <td>`+ parseInt(music.duration / 60) + `:` + music.duration % 60 + `s</td>
                        </tr>
                     `;
                    });

                    musicsTable += "</tbody></table></div>";

                    form.find(".row").append(musicsTable);
                    setTimeout(function () {
                        $('#tableMusic').DataTable({
                            "info": false,
                            "binfo": false,
                            "bLengthChange": false,
                            "pageLength": 5
                        });
                        setTimeout(function () {
                            $(".paging_simple_numbers").parent().addClass("col-lg-12");
                            $("#tableMusic_filter").parent().addClass("col-lg-12");
                        }, 300);
                    }, 300);
                    break;

                case "photo":
                    var photoTable = `
                        <div class="col-lg-12" style="margin-top: 25px;">
                        <table style="background: transparent; color:white;" id="tablePhoto" class="table table-bordered">
                            <thead>
                                <tr>
                                    <th></th>
                                    <th>Image</th>
                                </tr>
                            </thead>
                            <tbody>
                    `;

                    $.each(photos, function (index, photo) {
                        photoTable += `
                        <tr>
                            <td><input type="checkbox" class="check check-photo" data-id="` + photo.id + `"</td>
                            <td><img src="/uploads/photos/`+ photo.path + `" width="48px"/></td>
                        </tr>
                     `;
                    });

                    photoTable += "</tbody></table></div>";

                    form.find(".row").append(photoTable);
                    setTimeout(function () {
                        $('#photoTable').DataTable();
                        setTimeout(function () {
                            $(".dataTables_info").parent().remove();
                            $(".dataTables_length").parent().remove();
                            $(".paging_simple_numbers").parent().addClass("col-lg-12");
                            $("#tablePhoto_filter").parent().addClass("col-lg-12");
                        }, 300);
                        $('.check-photo').off("change").change(function (e) {
                            e.preventDefault();
                            var checked = $(this).is(":checked");
                            $(".check-photo:checked").prop("checked", false);
                            $(this).prop("checked", checked);
                        })
                    }, 300);
                    break;

                case "album":
                    var albumTable = `
                        <div class="col-lg-12" style="margin-top: 25px;">
                        <table style="background: transparent; color:white;" id="tableAlbum" class="table table-bordered">
                            <thead>
                                <tr>
                                    <th></th>
                                    <th></th>
                                    <th>Nom</th>
                                </tr>
                            </thead>
                            <tbody>
                    `;

                    $.each(albums, function (index, album) {
                        albumTable += `
                        <tr>
                            <td><input type="checkbox" class="check check-album" data-name="`+ album.name + `" data-id="` + album.id + `"</td>
                            <td><img src="/uploads/covers/`+ album.cover + `" width="48px"/></td>
                            <td>`+ album.name + `</td>
                        </tr>
                     `;
                    });

                    musicsTable += "</tbody></table></div>";

                    form.find(".row").append(albumTable);
                    setTimeout(function () {
                        $('#tableAlbum').DataTable();
                        setTimeout(function () {
                            $(".dataTables_info").parent().remove();
                            $(".dataTables_length").parent().remove();
                            $(".paging_simple_numbers").parent().addClass("col-lg-12");
                            $("#tableAlbum_filter").parent().addClass("col-lg-12");
                        }, 300);
                    }, 300);
                    break;
            }

            $this.$modal.find('.delete-event').hide().end().find('.save-event').show().end().find('.modal-body').empty().prepend(form).end().find('.save-event').unbind('click').click(function () {
                form.submit();
            });
            $this.$modal.find('form').on('submit', function () {
                var date = new Date(start);

                if ($(".check:checked").length) {
                    if((nbPubs[dateStr+"#"+$(".page.selected").data("id")] + $(".check:checked").length) > 10) {
                        alert("Il ne reste que "+ (10 - nbPubs[dateStr+"#"+$(".page.selected").data("id")])+" publicité disponible");
                        return false;
                    }
                    $.each($(".check:checked"), function (index, element) {
                        var title = $(this).data("name");

                        var beginning = form.find("input[name='beginning']").val();
                        var ending = form.find("input[name='ending']").val();
                        var categoryClass = form.find("select[name='category'] option:checked").val();

                        if (type == "music" && $(".check-music:checked").length == 0) {
                            alert("Veuiller selectionner un music");
                            return false;
                        }

                        if (type == "album" && $(".check-album:checked").length == 0) {
                            alert("Veuiller selectionner un album");
                            return false;
                        }

                        if (type == "photo" && $(".check-photo:checked").length == 0) {
                            alert("Veuiller selectionner une photo");
                            return false;
                        }

                        if (title !== null && title.length != 0) {
                            var d = new Date();
                            var nbCreditPub = 0;
                            switch (type) {
                                case 'music':
                                    nbCreditPub = --nbPubCreditMusic;
                                    break;
                            
                                case 'album':
                                    nbCreditPub = --nbPubCreditsAlbum;
                                    break;
                                
                                case 'profile':
                                    nbCreditPub = --nbPubCreditProfile;
                                    break;

                                case 'photo':
                                    nbCreditPub = --nbPubCreditPhoto;
                                    break;
                            }
                            pubManager.addPub({
                                id: "_"+d.getTime(),
                                name: title,
                                price: (nbCreditPub && (nbCreditPub + 1) >0) ? 0 : selected.data("price"),
                                place: selected.data("place"),
                                date: date.getDate() + "/" + (date.getMonth() + 1) + "/" + date.getFullYear(),
                                dateEn: date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate(),
                                music: $(element).data("id"),
                                album: $(element).data("id"),
                                photo: $(element).data("id"),
                                pageId: selected.data("id"),
                                type: type
                            });

                            $this.$modal.modal('hide');
                        }
                        else {
                            alert('You have to give a title to your event');
                        }
                    });
                } else {
                    var d = new Date();
                    var nbCreditPub = --nbPubCreditProfile;
                    pubManager.addPub({
                        id: "_"+d.getTime(),
                        name: "Publicité profile",
                        price: (nbCreditPub && (nbCreditPub + 1) > 0 ) ? 0 : selected.data("price"),
                        place: selected.data("place"),
                        date: date.getDate() + "/" + (date.getMonth() + 1) + "/" + date.getFullYear(),
                        dateEn: date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate(),

                        pageId: selected.data("id"),
                        type: type
                    });

                    $this.$modal.modal('hide');
                }

                return false;

            });
            $this.$calendarObj.fullCalendar('unselect');
        }

    /* Initializing */
    CalendarApp.prototype.init = function () {
        /*  Initialize the calendar  */
        var date = new Date();
        var d = date.getDate();
        var m = date.getMonth();
        var y = date.getFullYear();
        var form = '';
        var today = new Date($.now());

        var defaultEvents = [];
        
        var $this = this;
        $this.$calendarObj = $this.$calendar.fullCalendar({
            slotDuration: '00:15:00', /* If we want to split day time each 15minutes */
            minTime: '08:00:00',
            maxTime: '19:00:00',
            defaultView: 'month',
            handleWindowResize: true,

            header: {
                left: 'prev,next today',
                center: 'title',
                right: ''
            },
            events: defaultEvents,
            editable: true,
            droppable: false, // this allows things to be dropped onto the calendar !!!
            eventLimit: true, // allow "more" link when too many events
            selectable: true,
            select: function (start, end, allDay) { $this.onSelect(start, end, allDay); },
            eventClick: function (calEvent, jsEvent, view) { $this.onEventClick(calEvent, jsEvent, view); },
            viewRender: function () {
                $("td.fc-widget-content[data-date]").css("position", "relative");
                $.each($("td.fc-widget-content[data-date]"), function (index, element) { 
                     var date = new Date($(element).data("date"));
                     var now = new Date();
                     now.setDate(now.getDate() - 1);
                     if(date >= now){
                         $(element).append("<b class='nbPub'>10</b>");
                     }
                    });
                pubManager.update();
            }
        });

        //on new event
        this.$saveCategoryBtn.on('click', function () {
            var categoryName = $this.$categoryForm.find("input[name='category-name']").val();
            var categoryColor = $this.$categoryForm.find("select[name='category-color']").val();
            if (categoryName !== null && categoryName.length != 0) {
                $this.$extEvents.append('<div class="calendar-events bg-' + categoryColor + '" data-class="bg-' + categoryColor + '" style="position: relative;"><i class="fa fa-move"></i>' + categoryName + '</div>');
            }

        });
    },

        //init CalendarApp
        $.CalendarApp = new CalendarApp, $.CalendarApp.Constructor = CalendarApp

}(window.jQuery),

    //initializing CalendarApp
    function ($) {
        "use strict";
        $.CalendarApp.init()
    }(window.jQuery);